# FastApi et upload de fichier

![tag](fastapi)
![category](python)

Un petit exemple de code permettant d'uploader un fichier via une api

on pourra tuiliser la version gui sur http://127.0.0.1:8080 ou via la doc http://127.0.0.1:8080/docs

le fichier main.py

```python
from fastapi import FastAPI
from api import router as routerapi
import gui
from codeserver import codeServerMiddleware

app = FastAPI()
app.add_middleware(codeServerMiddleware(prefix=""))
app.include_router(routerapi)
gui.init(app)


if __name__ == "__main__":
    import uvicorn
    uvicorn.run("main:app", port=8080, reload=True)
```

le fichier deps.py

```python
import httpx
from nicegui import ui


class NotifyingHttpClient:
    def __init__(self, ):
        self.client = None

    async def __aenter__(self):
        self.client = httpx.AsyncClient(follow_redirects=True)
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.client.aclose()

    async def get(self, url, **kwargs):
        return await self._request(self.client.get, url, **kwargs)

    async def post(self, url, **kwargs):
        return await self._request(self.client.post, url, **kwargs)

    async def put(self, url, **kwargs):
        return await self._request(self.client.put, url, **kwargs)

    async def delete(self, url, **kwargs):
        return await self._request(self.client.delete, url, **kwargs)

    async def _request(self, method, url, **kwargs):
        response = await method(url, **kwargs)
        if response.status_code > 400:
            ui.notify(
                f"Erreur {response.status_code} sur {url}",
                type='negative'
            )
        return response

```

le fichier gui.py

```python
from fastapi import FastAPI, File, UploadFile, Request
from io import BytesIO
from nicegui import ui
import httpx
from .deps import NotifyingHttpClient

def init(app):
	@ui.page("/")
	async def upload(request: Request):
	    ui.label("Uploader un fichier vers l'API")

	    async def envoyer_fichier(uploaded_file):
		    content = uploaded_file.content.read()
		    file_name = uploaded_file.name
		    file_type = uploaded_file.type
		    
            async with NotifyingHttpClient() as client:
                response = await client.post('http://127.0.0.1:8080/uploadfile',
                    files={"file": (file_name, BytesIO(content), file_type)})
                if response.status_code == 200:
                    ui.notify(f"✅ Fichier envoyé")
                    print(response.json())
                else:
                    ui.notify("❌ Erreur lors de l'envoi")

	    ui.upload(
		on_upload=envoyer_fichier,
		multiple=False,
		auto_upload=True
	    )

	ui.run_with(
		app,
		mount_path='/',
		storage_secret='my-secret',
	)
```

le fichier api.py

```python
from fastapi import FastAPI, File, UploadFile, APIRouter

from io import BytesIO

router = APIRouter()

# Endpoint FastAPI pour upload de fichier
@router.post("/uploadfile")
async def upload_file(file: UploadFile = File(...)):
    contents = await file.read()
    return {
        "filename": file.filename,
        "content_type": file.content_type,
        "size": len(contents)
    }
    
@router.get("/version")
async def version():
    return {
        "version": "test"
    }
```

le fichier codeserver.py

```python
import os
from starlette.middleware.base import BaseHTTPMiddleware
from fastapi import Request


def codeServerMiddleware(prefix='127.0.0.1:8080'):
    CODESERVER_PREFIX = prefix

    class RootPathMiddleware(BaseHTTPMiddleware):
        async def dispatch(self, request: Request, call_next):
            if not request.url.path.startswith('/_nicegui'):
                request.scope["root_path"] = CODESERVER_PREFIX
            response = await call_next(request)
            return response

    return RootPathMiddleware
```

en prime un fichier pour ls test testmain.py

```python
import unittest
from fastapi.testclient import TestClient
from main import app
from io import BytesIO

class TestFileUpload(unittest.TestCase):
    def setUp(self):
        self.client = TestClient(app)

    def test_upload_file(self):
        file_content = b"Hello, this is a test file."
        file = BytesIO(file_content)
        response = self.client.post(
            "/uploadfile",
            files={"file": ("test.txt", file, "text/plain")}
        )

        self.assertEqual(response.status_code, 200)
        data = response.json()
        self.assertEqual(data["filename"], "test.txt")
        self.assertEqual(data["content_type"], "text/plain")
        self.assertEqual(data["size"], len(file_content))

if __name__ == "__main__":
    unittest.main()
```

